
# q-expansion of newform 2070.2.a.b, downloaded from the LMFDB on 13 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 2070
weight = 2
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[-1], [0], [-1], [0], [-4], [-2], [6], [4], [1], [2], [0], [-2], [-10], [-4], [0], [-6], [4], [-10], [-12], [8], [10], [-8], [4], [-18], [2], [-6], [0], [12], [-10], [6], [-16], [-20], [-2], [-20], [10], [8], [-10], [20], [-8], [-14], [-4], [-2], [0], [2], [-6], [-16], [4], [16], [20], [14], [6], [-16], [18], [-20], [14], [24], [-14], [0], [-10], [-18], [-4], [-22], [4], [-8], [10], [-30], [28], [2], [-12], [14], [-34], [-8], [-8], [-18], [12], [32], [-38], [-2], [-10], [26], [-12], [-2], [32], [34], [-8], [4], [30], [10], [18], [16], [-12], [-32], [40], [4], [20], [40], [34], [14], [-36], [14], [4], [-14], [36], [-18], [-4], [2], [-28], [30], [40], [-38], [-16], [-18], [-18], [-4], [16], [-26], [20], [-8], [-14], [20], [-50], [2], [-38], [36], [-28], [50], [-34], [32], [16], [6], [-12], [24], [-40], [-18], [22], [-14], [-54], [4], [34], [-26], [28], [42], [-40], [12], [46], [-24], [6], [-42], [44], [32], [-50], [-10], [20], [24], [44], [-32], [-32], [30], [-22], [-14], [-4], [-2], [-24], [-52], [-42], [-24], [0], [38], [-46], [10], [-20], [46], [8], [26], [-32], [30], [44], [-38], [-32], [54], [0], [36], [-26], [22], [-16], [-22], [14], [52], [42], [0], [34], [-28], [20], [-30], [52], [-34], [-14], [-26], [-26], [8], [-46], [64], [-42], [18], [-4], [-30], [-40], [-36], [54], [-52], [50], [42], [-64], [12], [72], [-38], [0], [30], [24], [66], [70], [32], [-18], [-56], [12], [-42], [62], [0], [-48], [-12], [-50], [-60], [40], [-26], [44], [-48], [-14], [-22], [-12], [56], [36], [-4], [40], [-18], [30], [8], [16], [-44], [-4], [-32], [-42], [22], [-40], [58], [-78], [12], [-34], [12], [-38], [-6], [-8], [-12], [22], [6], [14], [20], [-46], [30], [-68], [-70], [62], [4], [10], [8], [50], [-72], [68], [-2], [42], [52], [32], [32], [-8], [-66], [28], [0], [50], [42], [-40], [-50], [-46], [-60], [70], [12], [14], [34], [-8], [42], [36], [-76], [26], [18], [-56], [44], [12], [-30], [-44], [86], [-8], [22], [16], [-70], [54], [-12], [-72], [-70], [12], [0], [-14], [46], [84], [-22], [-30], [48], [-82], [-78], [-60], [12], [-80], [58], [-2], [2], [32], [12], [28], [52], [38], [-74], [42], [-56], [-26], [30], [-6], [64], [-46], [-4], [-90], [-68], [32], [-6], [-76], [74], [-78], [56], [14], [-90], [48], [60], [78], [56], [14], [70], [-80], [-68], [84], [-86], [18], [-16], [-22], [-20], [-20], [48], [26], [-80], [46], [-100], [48], [34], [6], [-6], [-30], [30], [40], [46], [20], [-24], [0], [-74], [-84], [0], [82], [-70], [20], [52], [24], [-38], [-88], [78], [12], [-54], [-34], [62], [24], [62], [-38], [-8], [-58], [62], [20], [-20], [82], [10], [-68], [4], [58], [50], [64], [72], [-42], [56], [-30], [-2], [-96], [-28], [10], [-46], [4], [-26], [-20], [40]]
